package de.kif.frontend.views.calendar

import de.kif.common.CALENDAR_GRID_WIDTH
import de.kif.frontend.iterator
import de.kif.frontend.launch
import de.kif.frontend.repository.ScheduleRepository
import de.westermann.kwebview.View
import de.westermann.kwebview.createHtmlView
import org.w3c.dom.HTMLAnchorElement
import org.w3c.dom.HTMLElement
import org.w3c.dom.events.EventListener

class CalendarTools(entry: CalendarEntry, view: HTMLElement) : View(view) {

    init {
        var linkM10: HTMLAnchorElement? = null
        var linkM5: HTMLAnchorElement? = null
        var linkReset: HTMLAnchorElement? = null
        var linkP5: HTMLAnchorElement? = null
        var linkP10: HTMLAnchorElement? = null
        var linkDel: HTMLAnchorElement? = null

        for (element in html.children) {
            when {
                element.classList.contains("calendar-tools-m10") -> linkM10 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-m5") -> linkM5 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-reset") -> linkReset = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-p5") -> linkP5 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-p10") -> linkP10 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-del") -> linkDel = element as? HTMLAnchorElement
            }
        }

        linkM10 = linkM10 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-m10")
            link.textContent = "-10"
            html.appendChild(link)
            link
        }
        linkM10.removeAttribute("href")
        linkM10.addEventListener("click", EventListener {
            entry.pending = true
            launch {
                val s = entry.schedule.get()
                ScheduleRepository.update(s.copy(time = s.time - 10))
            }
        })

        linkM5 = linkM5 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-m5")
            link.textContent = "-5"
            html.appendChild(link)
            link
        }
        linkM5.removeAttribute("href")
        linkM5.addEventListener("click", EventListener {
            entry.pending = true
            launch {
                val s = entry.schedule.get()
                ScheduleRepository.update(s.copy(time = s.time - 5))
            }
        })

        linkReset = linkReset ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-reset")
            link.textContent = "reset"
            html.appendChild(link)
            link
        }
        linkReset.removeAttribute("href")
        linkReset.addEventListener("click", EventListener {
            entry.pending = true
            launch {
                val s = entry.schedule.get()
                ScheduleRepository.update(s.copy(time = s.time / CALENDAR_GRID_WIDTH * CALENDAR_GRID_WIDTH))
            }
        })

        linkP5 = linkP5 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-p5")
            link.textContent = "+5"
            html.appendChild(link)
            link
        }
        linkP5.removeAttribute("href")
        linkP5.addEventListener("click", EventListener {
            entry.pending = true
            launch {
                val s = entry.schedule.get()
                ScheduleRepository.update(s.copy(time = s.time + 5))
            }
        })

        linkP10 = linkP10 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-p10")
            link.textContent = "+10"
            html.appendChild(link)
            link
        }
        linkP10.removeAttribute("href")
        linkP10.addEventListener("click", EventListener {
            entry.pending = true
            launch {
                val s = entry.schedule.get()
                ScheduleRepository.update(s.copy(time = s.time + 10))
            }
        })

        linkDel = linkDel ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-del")
            link.textContent = "del"
            html.appendChild(link)
            link
        }
        linkDel.removeAttribute("href")
        linkDel.addEventListener("click", EventListener {
            entry.pending = true
            launch {
                ScheduleRepository.delete(entry.scheduleId)
            }
        })
    }
}