package de.kif.backend

import com.uchuhimo.konf.Config
import com.uchuhimo.konf.ConfigSpec
import com.uchuhimo.konf.Item
import java.io.FileNotFoundException
import java.nio.file.Paths
import java.text.SimpleDateFormat
import java.util.*
import kotlin.reflect.KProperty

object Configuration {

    private val config: Config

    class ConfigDelegate<T>(val item: Item<T>) {
        operator fun getValue(thisRef: Any?, property: KProperty<*>): T {
            return config[item]
        }
    }

    fun <T> c(item: Item<T>) = ConfigDelegate(item)

    private object ServerSpec : ConfigSpec("server") {
        val host by required<String>()
        val port by required<Int>()
    }

    object Server {
        val host by c(ServerSpec.host)
        val port by c(ServerSpec.port)
    }

    private object PathSpec : ConfigSpec("path") {
        val web by required<String>()
        val sessions by required<String>()
        val uploads by required<String>()
        val database by required<String>()
    }

    object Path {
        val web by c(PathSpec.web)
        val webPath: java.nio.file.Path by lazy { Paths.get(web).toAbsolutePath() }

        val sessions by c(PathSpec.sessions)
        val sessionsPath: java.nio.file.Path by lazy { Paths.get(sessions).toAbsolutePath() }

        val uploads by c(PathSpec.uploads)
        val uploadsPath: java.nio.file.Path by lazy { Paths.get(uploads).toAbsolutePath() }

        val database by c(PathSpec.database)
        val databasePath: java.nio.file.Path by lazy { Paths.get(database).toAbsolutePath() }
    }

    private object ScheduleSpec : ConfigSpec("schedule") {
        val reference by required<String>()
    }

    object Schedule {
        val reference by c(ScheduleSpec.reference)
        val referenceDate: Date by lazy { SimpleDateFormat("yyyy-MM-dd").parse(reference) }
    }

    private object SecuritySpec : ConfigSpec("security") {
        val sessionName by required<String>("session_name")
        val signKey by required<String>("sign_key")
    }

    object Security {
        val sessionName by c(SecuritySpec.sessionName)
        val signKey by c(SecuritySpec.signKey)
    }

    private object GeneralSpec : ConfigSpec("general") {
        val allowedUploadExtensions by required<String>("allowed_upload_extensions")
    }

    object General {
        val allowedUploadExtensions by c(GeneralSpec.allowedUploadExtensions)
        val allowedUploadExtensionSet by lazy {
            allowedUploadExtensions.split(",").map { it.trim().toLowerCase() }.toSet()
        }
    }

    init {
        var config = Config {
            addSpec(ServerSpec)
            addSpec(PathSpec)
            addSpec(ScheduleSpec)
            addSpec(SecuritySpec)
            addSpec(GeneralSpec)
        }.from.toml.resource("portal.toml")

        try {
            config = config.from.toml.file("portal.toml")
        } catch (_: FileNotFoundException) {
        }

        this.config = config.from.env()
            .from.systemProperties()
    }
}